"use strict";
//
//  This file is part of the Pale Moon Locale Switcher extension.
//  
//  Copyright  2014,2015 Jeremy Morton
//  Copyright  2016 Mark Straver BASc
//
//  The Pale Moon Locale Switcher extension is free software: you can
//  redistribute it and/or modify it under the terms of the GNU General
//  Public License as published by the Free Software Foundation, either
//  version 3 of the License, or (at your option) any later version.
//
//  The Pale Moon Locale Switcher extension is distributed in the hope that
//  it will be useful, but WITHOUT ANY WARRANTY; without even the implied
//  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

// Bootstrap file for Locale Switcher extension

// Components shortcuts
const Ci = Components.interfaces;
const Cc = Components.classes;
const Cr = Components.results;

function startup(data, reason) {
	// *** Called on extension startup ***
	let wm =  Cc["@mozilla.org/appshell/window-mediator;1"]
	          .getService(Ci.nsIWindowMediator);
	
	// If we, say, enable the addon for the first time when several browser windows are open, we want isFirstRun passed into SetupUiInBrowser for every
	// loop iteration so it's added to the nav bar each time; therefore we check once here for "is first run" and pass that in.
	let isFirstRun = bootstrapper.IsFirstRun();
	
	// Get the list of browser windows already open
	let windows = wm.getEnumerator("navigator:browser");
	while (windows.hasMoreElements()) {
		let domWindow = windows.getNext().QueryInterface(Ci.nsIDOMWindow);
		bootstrapper.SetupUiInBrowser(domWindow, isFirstRun);
	}
	
	// Wait for any new browser windows to open
	wm.addListener(bootstrapper);
	
	bootstrapper.RebuildLocales();
}

function shutdown(data, reason) {
	// *** Called on extension shutdown ***
	// When the application is shutting down we normally don't have to clean up any UI changes made
	if (reason == APP_SHUTDOWN) {
		return;
	}
	
	let wm =  Cc["@mozilla.org/appshell/window-mediator;1"]
	          .getService(Ci.nsIWindowMediator);
	
	// Get the list of browser windows already open
	let windows = wm.getEnumerator("navigator:browser");
	while (windows.hasMoreElements()) {
		let domWindow = windows.getNext().QueryInterface(Ci.nsIDOMWindow);
		bootstrapper.TeardownUiInBrowser(domWindow);
	}
	
	// As this won't be called on app shutdown or when a browser window closes - only on things like 'disable extension' or 'upgrade extension' - we should
	// unload bootstrapper's JSMs here to allow for them to potentially be upgraded or garbage collected.
	bootstrapper.UnloadJSMs();
	
	// Stop listening for any new browser windows to open
	wm.removeListener(bootstrapper);
}

function install(data, reason) {
	// *** Called on extension install ***
	// Because of bug #1008565, this function seems to get called at the wrong time and is unable to load the extension's JSMs.  For those reasons it's
	// basically useless and should do nothing; instead, check a 'first run' pref each time startup is called, and unless it exists and is false, run our
	// 'install' code then.
	// If/when that bug gets fixed, this function should check the 'reason' and if it's because of ADDON_INSTALL (as opposed to ADDON_UPGRADE or
	// ADDON_DOWNGRADE), it should do the 'install' code (add to end of #nav-bar).
}

function uninstall(data, reason) {
	// *** Called on extension uninstall ***
	let utilsJsm = {};
	
	// Because we can't use chrome:// URLs here (we can't guarantee that chrome.manifest has been processed yet for this extension), we have to determine
	// the full path to the extension's module file from data.resourceURI and load it that way.
	Components.utils.import(data.resourceURI.spec + "chrome/pm-localeswitch/content/modules/miscUtils.jsm", utilsJsm);
	bootstrapper.IsFirstRun("reset", utilsJsm);
}

// Provides a listener for language pack addons that are installed/uninstalled
var langPackListener = {
	onInstalled: function(addon) {
		bootstrapper.MaybeRebuildLocales(addon);
	}
	,
	onUninstalled: function(addon) {
		bootstrapper.MaybeRebuildLocales(addon);
	}
	,
	onEnabled: function(addon) {
		bootstrapper.MaybeRebuildLocales(addon);
	}
	,
	onDisabled: function(addon) {
		bootstrapper.MaybeRebuildLocales(addon);
	}
};

// Provides various pieces of bootstrap functionality, such as being a window listener that performs UI creation/teardown
var bootstrapper = (function(my) {
	
	// Private members (please don't use them directly from calling code)
	// ==================================================================
	
	my._custUI = null;
	
	
	
	// Public members
	// ==============
	
	my.ImportJSMs = function() {
		// *** Import JSMs into 'jsm' object ***
		
		my.jsm = my.jsm || {};
		// Customizable UI
		if (my._custUI != false) {
			my._custUI = my._custUI || false;
			try {
				Components.utils.import("resource:///modules/CustomizableUI.jsm", my.jsm);
				my._custUI = true;
			}
			catch(ex) {  }
		}
		// Addon manager
		Components.utils.import("resource://gre/modules/AddonManager.jsm", my.jsm);
		// Manifest parser
		Components.utils.import("resource://gre/modules/ChromeManifestParser.jsm", my.jsm);
		// Dynamic UI
		Components.utils.import("chrome://pm-localeswitch/content/modules/dynamicUI.jsm", my.jsm);
		// Misc utilities
		Components.utils.import("chrome://pm-localeswitch/content/modules/miscUtils.jsm", my.jsm);
		// Toolbar button 'select locale'
		Components.utils.import("chrome://pm-localeswitch/content/toolbtnSelectLocale.jsm", my.jsm);
	};
	
	my.UnloadJSMs = function() {
		// *** Unload JSMs (only those specific to this extension) ***
		
		Components.utils.unload("chrome://pm-localeswitch/content/modules/dynamicUI.jsm");
		Components.utils.unload("chrome://pm-localeswitch/content/modules/miscUtils.jsm");
		Components.utils.unload("chrome://pm-localeswitch/content/toolbtnSelectLocale.jsm");
	};
	
	my.LastRecordedVersion = function(value) {
		my.ImportJSMs();
		
		if (value == null) {
			return my.jsm.miscUtils.PrefString({
				prefName: my.jsm.miscUtils.GetEscapedExtId() + "lastRecordedVersion"
			});
		}
		else {
			my.jsm.miscUtils.PrefString({ prefName: my.jsm.miscUtils.GetEscapedExtId() + "lastRecordedVersion", setTo: value });
		}
	};
	
	my.IsFirstRun = function(value, miscUtilsContainer) {
		// Gets/sets 'is first run' pref for this extension
		if (miscUtilsContainer == null) {
			my.ImportJSMs();
			miscUtilsContainer = {};
			miscUtilsContainer.miscUtils = my.jsm.miscUtils;
		}
		if (value == null) {
			return miscUtilsContainer.miscUtils.PrefBool({
				prefName: miscUtilsContainer.miscUtils.GetEscapedExtId() + "isFirstRun",
				defaultVal: true
			});
		}
		else if (value == "reset") {
			miscUtilsContainer.miscUtils.PrefBool({ prefName: miscUtilsContainer.miscUtils.GetEscapedExtId() + "isFirstRun", doReset: true });
		}
		else {
			miscUtilsContainer.miscUtils.PrefBool({ prefName: miscUtilsContainer.miscUtils.GetEscapedExtId() + "isFirstRun", setTo: value });
		}
	};
	
	my.SetupUiInBrowser = function(window, isFirstRun) {
		my.ImportJSMs();
		my.jsm.zinglocaleLib.InitJSM(my.jsm);
		
		let doc = window.document;
		
		// Cache addon info
		if (!my.jsm.miscUtils.DataStore("pm-localeswitchAddon")) {
			my.jsm.AddonManager.getAddonByID(my.jsm.miscUtils.GetExtId(), function(addon){
				my.jsm.miscUtils.DataStore("pm-localeswitchAddon", addon);
				
				// Extension upgraded?  If so, display changelog.
				let versionStr = "v" + my.jsm.miscUtils.DataStore("pm-localeswitchAddon").version;
				
				let lastRecVer = my.LastRecordedVersion();
				if (lastRecVer == null || lastRecVer != versionStr) {
					// Extension appears to have been upgraded.
					my.LastRecordedVersion(versionStr);
					
					// NOTE: The tabbrowser seems to be broken and the following doesn't work properly when called on window load:
					// window.gBrowser.selectedTab = window.gBrowser.addTab(...);
					// So we'll just open a completely new browser window for the upgrade notice instead of a new tab.
					// my.jsm.miscUtils.OpenURL('http://gooeysoftware.com/mozaddons/zinglocale/changelog.php?upgrade=' + versionStr);
				}
			});
		}
		
		if (isFirstRun == null) {
			isFirstRun = my.IsFirstRun();
		}
		
		// If we're using CustomizableUI, we only want to setup the part of our UI we're using it for (our toolbar button) once per extension session
		if (!my._custUI || !my.jsm.miscUtils.DataStore("uiSetupThisSession")) {
			// Create toolbar button
			my.jsm.zinglocaleLib.toolbtnSelectLocale.UiSetup.CreateToolbtn(my._custUI, doc, window, isFirstRun);
			
			my.jsm.miscUtils.DataStore("uiSetupThisSession", true);
		}
		
		// Insert popup menu into the appropriate place in the Tools menu too
		my.jsm.zinglocaleLib.toolbtnSelectLocale.UiSetup.CreateMenuEntry(doc, window);
		
		if (isFirstRun) {
			bootstrapper.IsFirstRun(false);
		}
		
		// Add listener for any installs/uninstalls of language packs
		my.jsm.AddonManager.addAddonListener(langPackListener);
	};
	
	my.TeardownUiInBrowser = function(window) {
		my.ImportJSMs();
		my.jsm.zinglocaleLib.InitJSM(my.jsm);
		
		let doc = window.document;
		
		// If we're using CustomizableUI, we only want to teardown the part of our UI we're using it for (our toolbar button) once per extension session
		if (!my._custUI || my.jsm.miscUtils.DataStore("uiSetupThisSession")) {
			// Destroy toolbar button
			my.jsm.zinglocaleLib.toolbtnSelectLocale.UiSetup.DestroyToolbtn(my._custUI, doc);
			
			my.jsm.miscUtils.DataStore("uiSetupThisSession", false);
		}
		
		// Remove popup menu from the Tools menu too
		my.jsm.zinglocaleLib.toolbtnSelectLocale.UiSetup.DestroyMenuEntry(doc);
		
		// Remove listener for any installs/uninstalls of language packs
		my.jsm.AddonManager.removeAddonListener(langPackListener);
	};
	
	my.MaybeRebuildLocales = function(addon) {
		if (addon.type == "locale") { my.RebuildLocales(); }
	};
	
	my.RebuildLocales = function() {
		my.ImportJSMs();
		
		// Build up a list of locales and associated information on them in our data store
		let db = my.jsm.miscUtils.GetDefaultBranch("general.useragent.");
//		let defaultLocale = db.getCharPref("locale");
		let defaultLocale = "POSIX";
		my.jsm.AddonManager.getAddonsByTypes(['locale'], function(addons){
			let localesList = [];
			
			for (let i=0; i < addons.length; i++) {
				let thisAddon = addons[i];
				let localeName = "(UnknownLocale)";
				
				if (thisAddon.userDisabled) { continue; }
				
				// Parse addon's manifest
				let manifestURI = thisAddon.getResourceURI("chrome.manifest");
				if (manifestURI) {
					let manifest = my.jsm.ChromeManifestParser.parseSync(manifestURI);
					for (let j=0; j < manifest.length; j++) {
						let thisEntry = manifest[j];
						if (thisEntry.type == 'locale' && thisEntry.args[0] == 'global') {
							localeName = thisEntry.args[1];
							break;
						}
					}
				}
				
				localesList.push({
					localeName: localeName,
					friendlyLocaleName: my.GetFriendlyLocaleName(localeName),
					isDefaultLocale: localeName == defaultLocale,
					langpackFound: true,
					langpackName: thisAddon.name,
					langpackVersion: thisAddon.version,
					appVersion: my.jsm.miscUtils.AppVersion(),
					// SeaMonkey language pack minVersion and maxVersion cannot be trusted; assume a problematic version mismatch if
					// application is SeaMonkey and language pack version differs from application version.
					versionMismatch:
						my.jsm.miscUtils.AppName() == "recognized:sm" ?
						my.jsm.miscUtils.CompareAppVersions(thisAddon.version, my.jsm.miscUtils.AppVersion()) != 0 :
						false
				});
			}
			
			// Try to marry up our existing locales data with the chrome registry of 'global' locales
			let chromeReg =  Cc["@mozilla.org/chrome/chrome-registry;1"]
			                 .getService(Ci.nsIToolkitChromeRegistry);
			let availableLocales = chromeReg.getLocalesForPackage("global");
			while (availableLocales.hasMore()) {
				let locale = availableLocales.getNext();
				
				let localeFound = false;
				for (let i=0; i < localesList.length; i++) {
					if (localesList[i].localeName == locale) { localeFound = true; break; }
				}
				
				if (!localeFound) {
					localesList.push({
						localeName: locale,
						friendlyLocaleName: my.GetFriendlyLocaleName(locale),
						isDefaultLocale: locale == defaultLocale,
						langpackFound: false,
						langpackName: null,
						langpackVersion: null,
						appVersion: my.jsm.miscUtils.AppVersion(),
						versionMismatch: null
					});
				}
			}
			
			// Sort locales list with default(s) at top
			let sortedLocalesList = [];
			let sortedDefaultsList = [];
			for (let i=0; i < localesList.length; i++) {
				if (localesList[i].isDefaultLocale) {
					sortedDefaultsList.push(localesList[i]);
				}
				else {
					sortedLocalesList.push(localesList[i]);
				}
			}
			sortedDefaultsList.sort(function(a, b){
				// Sort in reverse as we're going to unshift them into the main array
				return a.localeName < b.localeName;
			});
			sortedLocalesList.sort(function(a, b){
				return a.localeName > b.localeName;
			});
			for (let i=0; i < sortedDefaultsList.length; i++) {
				sortedLocalesList.unshift(sortedDefaultsList[i]);
			}
			
			my.jsm.miscUtils.DataStore("localesList", sortedLocalesList);
		});
	};
	
	my.GetFriendlyLocaleName = function(locale) {
		let langNames = my.jsm.miscUtils.GetStrBundle("chrome://global/locale/languageNames.properties");
		let regNames = my.jsm.miscUtils.GetStrBundle("chrome://global/locale/regionNames.properties");
		let parts = locale.split(/-/);
		let friendlyName;
		
		try {
			friendlyName = langNames.GetStringFromName(parts[0]);
			if (parts.length > 1) {
				try {
					friendlyName += " (" + regNames.GetStringFromName(parts[1].toLowerCase()) + ")";
				}
				catch(ex) {
					friendlyName += " (" + parts[1] + ")";
				}
			}
			
			friendlyName = locale + " - " + friendlyName;
		}
		catch(ex) {
			return locale;
		}
		
		return friendlyName;
	};
	
	// nsIWindowMediatorListener implementation
	my.onOpenWindow = function(xulWindow) {
		my.ImportJSMs();
		
		// A new window has opened
		let domWindow = xulWindow.QueryInterface(Ci.nsIInterfaceRequestor)
		                .getInterface(Ci.nsIDOMWindow);
		
		// Wait for it to finish loading
		domWindow.addEventListener("load", function listener() {
			domWindow.removeEventListener("load", listener, false);
			
			// If this is a browser window then setup our UI
			if (domWindow.document.documentElement.getAttribute("windowtype") == "navigator:browser") {
				bootstrapper.SetupUiInBrowser(domWindow);
			}
		}, false);
	};
	
	my.onCloseWindow = function(xulWindow) { };
	
	my.onWindowTitleChange = function(xulWindow, newTitle) { };
	
	// *** Finally ***
	// Return |my| - everything we've assigned to it will be exported to the public var we're defining.
	return my;
	
}(bootstrapper || {}));
