"use strict";

/*
    This file is part of the Pale Moon Locale Switcher extension.
    
    Copyright (C) 2014,2015 Jeremy Morton
    Copyright (C) 2016 Mark Straver BASc

    The Pale Moon Locale Switcher extension is free software: you can
    redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either
    version 3 of the License, or (at your option) any later version.

    The Pale Moon Locale Switcher extension is distributed in the hope that
    it will be useful, but WITHOUT ANY WARRANTY; without even the implied
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// dlgLanguages.js - Languages for web content configuration dialog utilities
// Provides module definition for web content preferred languages dialog functionality.

// Components shortcuts
const Ci = Components.interfaces;
const Cc = Components.classes;
const Cr = Components.results;

var zinglocaleLib = (function(my) {
	
	// *** Import JSMs into 'jsm' object ***
	my.jsm = my.jsm || {};
	// Misc utilities
	Components.utils.import("chrome://pm-localeswitch/content/modules/miscUtils.jsm", my.jsm);
	// Web content languages functionality
	Components.utils.import("chrome://pm-localeswitch/content/modules/webContentLang.jsm", my.jsm);
	
	// *** Private vars ***
	// Grab string bundle object
	var bundle = my.jsm.miscUtils.GetStrBundle("chrome://pm-localeswitch/locale/pm-localeswitch.properties");
	
	// *** Main functionality ***
	my.dlgLanguages = my.dlgLanguages || {};
	
	my.dlgLanguages.OnLoad = function() {
		var thisChromeWindow = document.defaultView;
		
		// Insert 'Apply' before Cancel, and 'OK' before 'Apply', to get 'OK / Apply / Cancel' buttons in dialog corner.
		// document.documentElement will get the root node.  Must use this in the case of a dialog; with a window, the
		// variable 'window' is directly available to the JS code.  With a dialog, no 'dialog' equivalent is available.
		// getButton is a function that gets the button with the specified 'type', which corresponds to one of the legal
		// values that can be specified in the list for the dialog 'buttons' attribute.
		var btnCancel = document.documentElement.getButton("cancel");
		var btnApply = document.getElementById("btnApply");
		var btnOk = document.getElementById("btnOk");
		
		btnCancel.parentNode.insertBefore(btnApply, btnCancel);
		btnCancel.parentNode.insertBefore(btnOk, btnApply);
		btnApply.setAttribute("label", bundle.GetStringFromName("generalBtnApply"));
		btnApply.setAttribute("disabled", "true");
		btnOk.setAttribute("label", bundle.GetStringFromName("generalBtnOk"));
		btnApply.hidden = false;
		btnOk.hidden = false;
		
		// Set available languages dropdown prompt text
		var availableLanguagesList = document.getElementById("lstAvailableLanguages");
		availableLanguagesList.setAttribute("label", bundle.GetStringFromName("languagesListSelectLanguage"));
		
		// Init module
		var prefsBundle = my.jsm.miscUtils.GetStrBundle("chrome://browser/locale/preferences/preferences.properties", true);
		if (prefsBundle == null) {
			// Try alternative "language coding preferences" bundle that SeaMonkey might use
			prefsBundle = my.jsm.miscUtils.GetStrBundle("chrome://communicator/locale/pref/pref-languages.properties");
		}
		my.jsm.webContentLang.Init(
			my.jsm.miscUtils.GetStrBundle("resource://gre/res/language.properties"),
			my.jsm.miscUtils.GetStrBundle("chrome://global/locale/regionNames.properties"),
			my.jsm.miscUtils.GetStrBundle("chrome://global/locale/languageNames.properties"),
			prefsBundle
		);
		
		// Setup "add language" dropdown
		var availableLanguages = my.jsm.webContentLang.GetLanguages();
		var availableLanguagesPopup = document.getElementById("popupAvailableLanguages");
		while (availableLanguagesPopup.hasChildNodes()) {
			availableLanguagesPopup.removeChild(availableLanguagesPopup.firstChild);
		}
		for (var i=0; i < availableLanguages.length; i++) {
			var thisLang = availableLanguages[i];
			
			if (thisLang.isVisible) {
				var menuitem = document.createElement("menuitem");
				menuitem.id = thisLang.abcd;
				availableLanguagesPopup.appendChild(menuitem);
				menuitem.setAttribute("label", thisLang.name);
			}
		}
		
		// Setup intl.accept_languages list
		var activeLanguagesList = document.getElementById("lstActiveLanguages");
		var prefValue = my.jsm.miscUtils.PrefString({
			prefName: "intl.accept_languages",
			mustExist: true,
			isLocalized: true
		});
		var languages = my.jsm.webContentLang.GetLanguagesFromCodesString(prefValue);
		for (var i=0; i < languages.length; i++) {
			var thisLang = languages[i];
			
			var listitem = document.createElement("listitem");
			listitem.id = thisLang.abcd;
			activeLanguagesList.appendChild(listitem);
			listitem.setAttribute("label", thisLang.name);
			
			// If found, hide from available languages popup
			for (var j=0; j < availableLanguagesPopup.children.length; j++) {
				if (availableLanguagesPopup.children[j].id == thisLang.abcd) {
					availableLanguagesPopup.children[j].hidden = true;
					break;
				}
			}
		}
		if (activeLanguagesList.childNodes.length > 0) {
			activeLanguagesList.selectedIndex = 0;
		}
	};
	
	my.dlgLanguages.OnLanguageSelect = function() {
		var upButton = document.getElementById("btnMoveUp");
		var downButton = document.getElementById("btnMoveDown");
		var removeButton = document.getElementById("btnRemove");
		var activeLanguagesList = document.getElementById("lstActiveLanguages");
		
		switch (activeLanguagesList.selectedCount) {
			case 0:
				upButton.disabled = downButton.disabled = removeButton.disabled = true;
				break;
			case 1:
				upButton.disabled = activeLanguagesList.selectedIndex == 0;
				downButton.disabled = activeLanguagesList.selectedIndex == activeLanguagesList.childNodes.length - 1;
				removeButton.disabled = false;
				break;
			default:
				upButton.disabled = true;
				downButton.disabled = true;
				removeButton.disabled = false;
				break;
		}
	};
	
	my.dlgLanguages.OnMoveUp = function() {
		var activeLanguagesList = document.getElementById("lstActiveLanguages");
		var newIndex = activeLanguagesList.selectedIndex - 1;
		activeLanguagesList.insertBefore(activeLanguagesList.selectedItems[0], activeLanguagesList.selectedItems[0].previousSibling);
		activeLanguagesList.ensureIndexIsVisible(newIndex);
		activeLanguagesList.selectedIndex = newIndex;
		
		my.dlgLanguages.EnableApplyBtn();
	};
	
	my.dlgLanguages.OnMoveDown = function() {
		var activeLanguagesList = document.getElementById("lstActiveLanguages");
		var newIndex = activeLanguagesList.selectedIndex + 1;
		activeLanguagesList.insertBefore(activeLanguagesList.selectedItems[0], activeLanguagesList.selectedItems[0].nextSibling.nextSibling);
		activeLanguagesList.ensureIndexIsVisible(newIndex);
		activeLanguagesList.selectedIndex = newIndex;
		
		my.dlgLanguages.EnableApplyBtn();
	};
	
	my.dlgLanguages.OnRemove = function() {
		var activeLanguagesList = document.getElementById("lstActiveLanguages");
		var selectedItems = [];
		for (var i=0; i < activeLanguagesList.children.length; i++) {
			if (activeLanguagesList.children[i].selected) { selectedItems.push(activeLanguagesList.children[i]); }
		}
		for (var i=0; i < selectedItems.length; i++) {
			activeLanguagesList.removeChild(selectedItems[i]);
		}
		
		my.dlgLanguages.EnableApplyBtn();
	};
	
	my.dlgLanguages.OnAvailableLanguageSelect = function() {
		document.getElementById("btnAdd").disabled = false;
	};
	
	my.dlgLanguages.OnAdd = function() {
		var activeLanguagesList = document.getElementById("lstActiveLanguages");
		var availableLanguagesList = document.getElementById("lstAvailableLanguages");
		
		// Add language to top of list and select it
		var listitem = document.createElement("listitem");
		listitem.id = availableLanguagesList.selectedItem.id;
		activeLanguagesList.insertBefore(listitem, activeLanguagesList.children[0]);
		listitem.setAttribute("label", availableLanguagesList.selectedItem.label);
		activeLanguagesList.selectedIndex = 0;
		
		// Hide added language from popup
		availableLanguagesList.selectedItem.hidden = true;
		
		// Disable 'add' button again, clear selection, and restore available languages dropdown prompt text
		document.getElementById("btnAdd").disabled = true;
		availableLanguagesList.selectedIndex = -1;
		availableLanguagesList.setAttribute("label", bundle.GetStringFromName("languagesListSelectLanguage"));
		
		my.dlgLanguages.EnableApplyBtn();
	};
	
	my.dlgLanguages.OnApply = function() {
		// Set intl.accept_languages pref from list
		var langString = my.jsm.webContentLang.GetCodesStringFromLanguages(document.getElementById("lstActiveLanguages").children);
		my.jsm.miscUtils.PrefString({
			prefName: "intl.accept_languages",
			mustExist: true,
			setTo: langString
		});
		
		// Finally, re-disable Apply button
		document.getElementById("btnApply").setAttribute("disabled", "true");
		
		return true;
	};
	
	my.dlgLanguages.OnOk = function() {
		// Clicking 'OK' is functionally identical to clicking 'Apply' and then 'Cancel', so let's do that.
		if (!document.getElementById("btnApply").getAttribute("disabled")) {
			if (my.dlgLanguages.OnApply()) {
				// Apply was successful; now close dialog.
				document.documentElement.getButton("cancel").click();
			}
		}
		else {
			// Apply wasn't necessary; close dialog.
			document.documentElement.getButton("cancel").click();
		}
	};
	
	my.dlgLanguages.EnableApplyBtn = function() {
		document.getElementById("btnApply").disabled = false;
	};
	
	// *** Finally ***
	// Return |my| - everything we've assigned to it will be exported to the public var we're defining.
	return my;
	
}(zinglocaleLib || {}));
