"use strict";
/*
    This file is part of the Pale Moon Locale Switcher extension.
    
    Copyright (C) 2014,2015 Jeremy Morton
    Copyright (C) 2016 Mark Straver BASc

    The Pale Moon Locale Switcher extension is free software: you can
    redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either
    version 3 of the License, or (at your option) any later version.

    The Pale Moon Locale Switcher extension is distributed in the hope that
    it will be useful, but WITHOUT ANY WARRANTY; without even the implied
    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
// toolbtnSelectLocale.jsm - Select locale button functionality module
// Contains code for implementing select locale button functionality.

var EXPORTED_SYMBOLS = ["zinglocaleLib"];

// Components shortcuts
const Ci = Components.interfaces;
const Cc = Components.classes;
const Cr = Components.results;

var zinglocaleLib = (function(my) {
	
	// Private members (please don't use them directly from calling code)
	// ==================================================================
	
	my._btnId = "zinglocaleToolbtnSelectLocale";
	my._menuId = "zinglocaleToolbarEntrySelectLocale";
	my._bundle = null;
	
	
	
	// Init - RUN FIRST TO INJECT MODULE DEPENDENCIES
	// ==============================================
	
	// Basically a fake constructor for JSM dependency injection - should be run before any other functions in this lib
	my.InitJSM = function(jsm) {
		// jsm should contain:
		// jsm.CustomizableUI
		// jsm.dynamicUI
		// jsm.miscUtils
		my.jsm = jsm;
	};
	
	
	
	// Public members
	// ==============
	
	// *** Main functionality ***
	my.toolbtnSelectLocale = my.toolbtnSelectLocale || {};
	
	my.toolbtnSelectLocale.OnPopupShowing = function(menuPopup, doc) {
		var thisChromeWindow = doc.defaultView;
		
		my.toolbtnSelectLocale.UiSetup.InitStringBundle();
		
		let chromeReg =  Cc["@mozilla.org/chrome/chrome-registry;1"]
		                 .getService(Ci.nsIToolkitChromeRegistry);
		let currentLocale = my.jsm.miscUtils.PrefString({
			prefName: "general.useragent.locale",
			defaultVal: "en-US"
		});
		let localesList = my.jsm.miscUtils.DataStore("localesList");
		
		// Empty current popup list
		let currentItems = menuPopup.childNodes;
		for (let i=currentItems.length; i > 0; i--) {
			menuPopup.removeChild(currentItems[i-1]);
		}
		
		let newItem;
		if (my.jsm.miscUtils.AppName() == "recognized:ff") {
			newItem = doc.createElement("menuitem");
			newItem.setAttribute("label", my._bundle.GetStringFromName("dropdownEntryGetFirefoxPacks"));
			newItem.addEventListener("command", function(event) { let thisWin = event.target.ownerDocument.defaultView; thisWin.gBrowser.selectedTab = thisWin.gBrowser.addTab('https://addons.mozilla.org/en-US/firefox/language-tools/', null, null, null); }, false);
			menuPopup.appendChild(newItem);
			menuPopup.appendChild(doc.createElement("menuseparator"));
		}
		else if (my.jsm.miscUtils.AppName() == "recognized:sm") {
			newItem = doc.createElement("menuitem");
			newItem.setAttribute("label", my._bundle.GetStringFromName("dropdownEntryGetSeaMonkeyPacks"));
			newItem.addEventListener("command", function(event) { let thisWin = event.target.ownerDocument.defaultView; thisWin.gBrowser.selectedTab = thisWin.gBrowser.addTab('http://www.seamonkey-project.org/releases/#langpacks', null, null, null); }, false);
			menuPopup.appendChild(newItem);
			menuPopup.appendChild(doc.createElement("menuseparator"));
		}
		else if (my.jsm.miscUtils.AppName() == "recognized:pm") {
			newItem = doc.createElement("menuitem");
			newItem.setAttribute("label", my._bundle.GetStringFromName("dropdownEntryGetPaleMoonPacks"));
			newItem.addEventListener("command", function(event) { let thisWin = event.target.ownerDocument.defaultView; thisWin.gBrowser.selectedTab = thisWin.gBrowser.addTab('https://www.palemoon.org/langpacks.shtml', null, null, null); }, false);
			menuPopup.appendChild(newItem);
			menuPopup.appendChild(doc.createElement("menuseparator"));
		}
		
		newItem = doc.createElement("menuitem");
		newItem.setAttribute("label", my._bundle.GetStringFromName("configurePreferredLocales"));
		newItem.addEventListener("command", function(event) { let thisWin = event.target.ownerDocument.defaultView; thisWin.openDialog('chrome://pm-localeswitch/content/dlgLanguages.xul', '', 'chrome,centerscreen,modal'); });
		menuPopup.appendChild(newItem);
		menuPopup.appendChild(doc.createElement("menuseparator"));
		
		newItem = doc.createElement("menuitem");
		newItem.setAttribute("disabled", "true");
		newItem.setAttribute("label", my._bundle.GetStringFromName("selectUiLocale"));
		menuPopup.appendChild(newItem);
		
		for (let i=0; i < localesList.length; i++) {
			let thisLocale = localesList[i];
			
			let showNoLangPackWarning = false;
			let showVersionMismatchWarning = false;
			let suffix = "";
			// Default locale probably won't be installed as a language pack, so doesn't deseve the 'noLangPack' suffix.
			if (!thisLocale.isDefaultLocale && !thisLocale.langpackFound) { suffix += " [" + my._bundle.GetStringFromName("localeNoLangPackSuffix") + "]"; showNoLangPackWarning = true; }
			if (thisLocale.versionMismatch) { suffix += " [" + my._bundle.GetStringFromName("localeVersionMismatchUppercaseSuffix") + "]"; showVersionMismatchWarning = true; }
			
			newItem = doc.createElement("menuitem");
			newItem.setAttribute("value", thisLocale.localeName);
			newItem.setAttribute("label", thisLocale.friendlyLocaleName + suffix);
			newItem.setAttribute("type", "radio");
			if (thisLocale.isDefaultLocale) {
				newItem.setAttribute("style", "font-weight:bold;");
			}
			if (currentLocale == thisLocale.localeName) {
				newItem.setAttribute("checked", "true");
			}
			newItem.addEventListener("command", function(event) { zinglocaleLib.toolbtnSelectLocale.SwitchToLocale(showNoLangPackWarning, showVersionMismatchWarning, event, event.target.ownerDocument.defaultView, thisLocale.langpackVersion, thisLocale.appVersion); }, false);
			menuPopup.appendChild(newItem);
		}
	};
	
	my.toolbtnSelectLocale.SwitchToLocale = function(showNoLangPackWarning, showVersionMismatchWarning, event, chromeWindow, langPackVersion, appVersion) {
		let selectedLocale = event.target.value;
		let selectedLocaleName = event.target.label;
		let currentLocale = my.jsm.miscUtils.PrefString({
			prefName: "general.useragent.locale",
			defaultVal: "en-US"
		});
		if (selectedLocale != currentLocale) {
			// Confirmation?
			if (
				showNoLangPackWarning &&
				!my.jsm.miscUtils.ModalConfirm(chromeWindow, my._bundle.GetStringFromName("localeNoLangPackTitle"), my._bundle.GetStringFromName("localeNoLangPackMessage"))
			) {
				return;
			}
			else if (
				showVersionMismatchWarning &&
				!my.jsm.miscUtils.ModalConfirm(chromeWindow, my._bundle.GetStringFromName("localeVersionMismatchTitle"), my._bundle.GetStringFromName("localeVersionMismatchMessage") + "\n- " + my._bundle.GetStringFromName("langPackVersion") + ": " + langPackVersion + "\n- " + my._bundle.GetStringFromName("appVersion") + ": " + appVersion)
			) {
				return;
			}
			
			// Do switch
			my.jsm.miscUtils.PrefString({ prefName: "general.useragent.locale", setTo: selectedLocale });
			
			let brandStrings = my.jsm.miscUtils.GetStrBundle("chrome://branding/locale/brand.properties");
			let extensionStrings = my.jsm.miscUtils.GetStrBundle("chrome://mozapps/locale/extensions/extensions.properties");
			let brandShortName = brandStrings.GetStringFromName("brandShortName");
			let promptStr = extensionStrings.formatStringFromName("notification.enable", [ selectedLocale, brandShortName ], 2);
			if (my.jsm.miscUtils.ModalConfirm(chromeWindow, null, promptStr)) {
				// Do restart to switch locale
				my.jsm.miscUtils.ShutdownApp(true);
			}
			else {
				// Set locale back to what it was before
				my.jsm.miscUtils.PrefString({ prefName: "general.useragent.locale", setTo: currentLocale });
			}
		}
	};
	
	// *** UI setup functionality ***
	my.toolbtnSelectLocale.UiSetup = my.toolbtnSelectLocale.UiSetup || {};
	
	my.toolbtnSelectLocale.UiSetup.InitStringBundle = function() {
		if (my._bundle == null) {
			// Grab string bundle object
			my._bundle = my.jsm.miscUtils.GetStrBundle("chrome://pm-localeswitch/locale/pm-localeswitch.properties");
		}
	};
	
	my.toolbtnSelectLocale.UiSetup.RegisterToolbtnSheet = function(doUnregister) {
		// Note that stuff like chrome://browser/skin/browser.css is registered at "author normal" (AUTHOR_SHEET) importance; see:
		// http://www.w3.org/TR/CSS21/cascade.html#cascading-order
		// In order for our rules to compete for specificity at the same level as the browser's theme stylings, ours needs to be registered with
		// AUTHOR_SHEET importance too; USER_SHEET stylesheets will be overridden by AUTHOR_SHEET stylesheets (unless !important is used).
		let ssSvc =  Cc["@mozilla.org/content/style-sheet-service;1"]
		             .getService(Ci.nsIStyleSheetService);
		let ioSvc =  Cc["@mozilla.org/network/io-service;1"]
		             .getService(Ci.nsIIOService);
		let uri = ioSvc.newURI("chrome://pm-localeswitch/skin/toolbtns.css", null, null);
		
		if (doUnregister) {
			// Unregister our toolbar button stylesheet globally
			if (ssSvc.sheetRegistered(uri, ssSvc.AUTHOR_SHEET)) {
				ssSvc.unregisterSheet(uri, ssSvc.AUTHOR_SHEET);
			}
		}
		else {
			// Register our toolbar button stylesheet globally
			if (!ssSvc.sheetRegistered(uri, ssSvc.AUTHOR_SHEET)) {
				ssSvc.loadAndRegisterSheet(uri, ssSvc.AUTHOR_SHEET);
			}
		}
	};
	
	my.toolbtnSelectLocale.UiSetup.CreateJsonTemplateBtn = function(doc, chromeWindow) {
		my.toolbtnSelectLocale.UiSetup.InitStringBundle();
		
		// Create JSON template for inserting our browser UI toolbar button
		let jsonTemplateBtn =
			[ "toolbarbutton",
				{
					id: my._btnId,
					class: "toolbarbutton-1",
					type: "menu",
					label: my._bundle.GetStringFromName("toolbtnSelectLocaleLabel"),
					tooltiptext: my._bundle.GetStringFromName("toolbtnSelectLocaleTip"),
					removable: true,
					key: "localeButton"
				},
				[ "menupopup",
					{
						onpopupshowing: function(event) { my.toolbtnSelectLocale.OnPopupShowing(this, doc); }
					},
					null
				]
			];
		return jsonTemplateBtn;
	};
	
	my.toolbtnSelectLocale.UiSetup.CreateToolbtn = function(custUI, doc, chromeWindow, isFirstRun) {
		let capturedNodes = {};
		
		// Register our stylesheet for this button
		my.toolbtnSelectLocale.UiSetup.RegisterToolbtnSheet();
		
		// Create JSON template for inserting our browser UI toolbar button
		let jsonTemplateBtn = my.toolbtnSelectLocale.UiSetup.CreateJsonTemplateBtn(doc, chromeWindow);
		
		if (custUI) {
			// CustomizableUI available
			
			// Create button menu dropdown widget and add it to the main toolbar (ie. #nav-bar)
			my.jsm.CustomizableUI.createWidget({
				id: my._btnId,
				type: "custom",
				defaultArea: my.jsm.CustomizableUI.AREA_NAVBAR,
				onBuild: function(doc) {
					return my.jsm.dynamicUI.JsonToDOM(jsonTemplateBtn, doc, capturedNodes);
				}
			});
		}
		else {
			// CustomizableUI not available
			
			// #navigator-toolbox and #nav-bar are elements that we can and will rely on the existence of, certainly in pre-Australis Firefox and SeaMonkey.
			// They even seem to still exist (for now) in the Firefox post-Australis interface, although we shouldn't need to use them there directly, as
			// we'll be using the CustomizableUI API instead (in fact, #nav-bar can't even be hidden anymore post-Australis!)
			// #nav-bar in particular is the best and most natural toolbar to insert our toolbarbutton into on installation, as it will always be available
			// even if it is hidden and all its toolbar items removed.  We can assume its existence in the browser UI and insert our button at the end of it
			// on first install.
			let toolbox = doc.getElementById("navigator-toolbox");
			if (toolbox == null) {
				my.jsm.miscUtils.ConsoleOutErr("Couldn't find #navigator-toolbox!  Can't setup toolbar UI.");
				return null;
			}
			let navbar = doc.getElementById("nav-bar");
			if (navbar == null) {
				my.jsm.miscUtils.ConsoleOutErr("Couldn't find #nav-bar!  Can't setup toolbar UI.");
				return null;
			}
			let palette = toolbox.palette;
			
			let domFragment = my.jsm.dynamicUI.JsonToDOM(jsonTemplateBtn, doc, capturedNodes);
			palette.appendChild(domFragment);
			
			// Look in the currentset of all toolbar's that are children of toolbox
			let foundInsert = false;
			for (let i=0; i < toolbox.childNodes.length; i++) {
				let thisToolbar = toolbox.childNodes[i];
				if (thisToolbar.localName == "toolbar" && thisToolbar.hasAttribute("currentset")) {
					let currentset = thisToolbar.getAttribute("currentset").split(",");
					let index = currentset.indexOf(my._btnId);
					if (index >= 0) {
						// The ID is in the currentset, so insert it at the appropriate index
						foundInsert = true;
						if (index >= thisToolbar.childNodes.length) {
							thisToolbar.insertItem(my._btnId);
						}
						else {
							thisToolbar.insertItem(my._btnId, thisToolbar.childNodes[index]);
						}
					}
				}
			}
			if (!foundInsert && isFirstRun) {
				navbar.insertItem(my._btnId);
				navbar.setAttribute("currentset", navbar.currentSet);
				doc.persist(navbar.id, "currentset");
			}
		}
	};
	
	my.toolbtnSelectLocale.UiSetup.CreateJsonTemplateMenu = function(doc, chromeWindow) {
		my.toolbtnSelectLocale.UiSetup.InitStringBundle();
		
		// Create JSON template for inserting our browser UI menu entry
		let jsonTemplateMenu =
			[ "menu",
				{
					id: my._menuId,
					label: my._bundle.GetStringFromName("toolbtnSelectLocaleLabel")
				},
				[ "menupopup",
					{
						onpopupshowing: function(event) { my.toolbtnSelectLocale.OnPopupShowing(this, doc); }
					},
					null
				]
			];
		return jsonTemplateMenu;
	};
	
	my.toolbtnSelectLocale.UiSetup.CreateMenuEntry = function(doc, chromeWindow) {
		let capturedNodes = {};
		
		let toolsPopup = doc.getElementById("taskPopup");                         // SeaMonkey
		if (!toolsPopup) { toolsPopup = doc.getElementById("menu_ToolsPopup"); }  // Firefox
		if (!toolsPopup) {
			my.jsm.miscUtils.ConsoleOutErr("Couldn't find Tools menu!  Can't setup toolbar UI.");
			return null;
		}
		let devToolsSeparator = doc.getElementById("devToolsSeparator");
		if (!devToolsSeparator) {
			my.jsm.miscUtils.ConsoleOutErr("Couldn't find Development Tools separator in Tools menu!  Can't setup toolbar UI.");
			return null;
		}
		
		// Create JSON template for inserting our browser UI menu entry
		let jsonTemplateMenu = my.toolbtnSelectLocale.UiSetup.CreateJsonTemplateMenu(doc, chromeWindow);
		
		let domFragment = my.jsm.dynamicUI.JsonToDOM(jsonTemplateMenu, doc, capturedNodes);
		// Note that |.insertBefore(newNode, referenceNode.nextSibling)| emulates |.insertAfter(newNode, referenceNode)|
		toolsPopup.insertBefore(domFragment, devToolsSeparator.nextSibling);
	};
	
	my.toolbtnSelectLocale.UiSetup.DestroyToolbtn = function(custUI, doc) {
		// Unregister our stylesheet
		my.toolbtnSelectLocale.UiSetup.RegisterToolbtnSheet(true);
		
		if (custUI) {
			// CustomizableUI available
			my.jsm.CustomizableUI.destroyWidget(my._btnId);
		}
		else {
			// CustomizableUI not available
			
			let toolbox = doc.getElementById("navigator-toolbox");
			if (toolbox == null) {
				my.jsm.miscUtils.ConsoleOutErr("Couldn't find #navigator-toolbox!  Can't teardown toolbar UI.");
				return null;
			}
			let palette = toolbox.palette;
			
			// Remove our toolbar button from toolbox, if it's been added there...
			if (!my.jsm.dynamicUI.RemoveToolbarItem(toolbox, my._btnId)) {
				// Couldn't find it in toolbox, try to remove it from palette
				if (!my.jsm.dynamicUI.RemoveChildNodeItem(palette, my._btnId)) {
					my.jsm.miscUtils.ConsoleOutErr("Couldn't find toolbar button to remove (with ID '" + my._btnId + "') in toolbox or palette!");
				}
			}
		}
	};
	
	my.toolbtnSelectLocale.UiSetup.DestroyMenuEntry = function(doc) {
		let toolsPopup = doc.getElementById("taskPopup");                         // SeaMonkey
		if (!toolsPopup) { toolsPopup = doc.getElementById("menu_ToolsPopup"); }  // Firefox
		if (!toolsPopup) {
			my.jsm.miscUtils.ConsoleOutErr("Couldn't find Tools menu!  Can't teardown toolbar UI.");
			return null;
		}
		let insertedMenu = doc.getElementById(my._menuId);
		if (!insertedMenu) {
			my.jsm.miscUtils.ConsoleOutErr("Couldn't find inserted Locale submenu in Tools menu!  Can't teardown toolbar UI.");
			return null;
		}
		toolsPopup.removeChild(insertedMenu);
	};
	
	// *** Finally ***
	// Return |my| - everything we've assigned to it will be exported to the public var we're defining.
	return my;
	
}(zinglocaleLib || {}));
